#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "csound.h"
#include "cscore.h"
#include "math.h"

int *pitchinputarray; //global to pass values to pitch functions
int *velocityinputarray; //global to pass values to velocity functions
float *rhyinputarray; //global pass values to rhythmic functions

//enter your path here to save files
char path[] = "/opt/csound516/";

int main(void)
{
    char c;
    int i, numpitches, numvelocities, numrhys;
    int *pitchreturnarray; //values returned from pitch functions
    int *pitchreturnarraysorted;
    int *velocityreturnarray; //values returned from velocity functions
    int *velocityreturnarraysorted;
    float *rhyreturnarray; //values returned from rhythmic functions
    float *p3array;
    int *p4array;

    CSOUND *cs;
    FILE    *insco, *outsco;
    int     result;
    const char *csvector[] = {"testin.sco", "testout.sco"};
    int argc = 2;

//****************************************
//********  CREATE PITCHES ************
//****************************************
    pitchreturnarray = pitchsetup(pitchinputarray);

//**************************************
//****  Count  number of pitches returned **********
//**************************************

    numpitches = 1;
    do
    {
        numpitches++;
    }
    while (pitchreturnarray[numpitches]>0);

//****************************************
//********  User Switch to Sort Pitches *****
//****************************************
userswitch:
    printf("\nSort Pitches? (y)es, (n)o: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        pitchreturnarraysorted = pitchsort(pitchreturnarray, numpitches);
        goto printsortpitch;
    case 'n':
        printf ("\n...no pitch sorting.");
        goto nosortpitch;
    default:
        printf("\n...enter (y)es, or (n)o: \n");
        goto userswitch;
    }
printsortpitch:

//**************************************
//********  PRINT PITCHES **************
//************* and ********************
//*****REPLACE ORIG WITH SORTED ********
//**************************************
    numpitches = 1;
    do
    {
        printf("%4d",pitchreturnarraysorted[numpitches]);
        numpitches++;
    }
    while (pitchreturnarraysorted[numpitches]>0);
    printf("\n");

    for (i=1; i<=numpitches-1; i++)
    {
        pitchreturnarray[i]=pitchreturnarraysorted[i];
    }
nosortpitch:

//************************************************
//********  User Switch to Create Velocities *****
//************************************************
velocitycreateswitch:
    printf("\n");
    printf("\nCreate velocity distribution? (y)es, (n)o: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        goto dovelocity;
    case 'n':
        velocityreturnarray = velocities_same(velocityinputarray);
        numvelocities = 1;
        do
        {
            numvelocities++;
        }
        while (velocityreturnarray[numvelocities]>0);
        goto novelocity;
    default:
        printf("\n...enter (y)es, or (n)o: \n");
        goto velocitycreateswitch;
    }
dovelocity:
//****************************************
//********  CREATE VELOCITIES ************
//****************************************
    velocityreturnarray = velocitysetup(velocityinputarray);

//*****************************************************
//****  Count  number of velocities returned **********
//*****************************************************
    numvelocities = 1;
    do
    {
        numvelocities++;
    }
    while (velocityreturnarray[numvelocities]>0);

//**********************************************
//********  User Switch to Sort Velocities *****
//**********************************************
velocitysortswitch:
    printf("\nSort Velocities? (y)es, (n)o: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        velocityreturnarraysorted = velocitysort(velocityreturnarray, numvelocities);
        goto printsortvelocity;
    case 'n':
        printf ("\n...no velocity sorting.");
        goto novelocity;
    default:
        break;
    }
printsortvelocity:

//*****************************************
//********  PRINT VELOCITIES **************
//************* and ***********************
//*****REPLACE ORIG WITH SORTED ***********
//*****************************************
    printf("\n...velocities from main.c\n"); //DEBUG
    numvelocities = 1;
    do
    {
        printf("%4d",velocityreturnarraysorted[numvelocities]);/*prints numbers */
        numvelocities++;
    }
    while (velocityreturnarraysorted[numvelocities]>0);
    printf("\n");

    for (i=1; i<=numvelocities-1; i++)
    {
        velocityreturnarray[i]=velocityreturnarraysorted[i];
    }
novelocity:

//**************************************
//********  GENERATE RHYTHM ************
//**************************************

    printf("\n\n");
    printf("\t\t\t***** RHYTHM DISTRIBUTIONS *****");
    printf("\n");
    int dims = 1;

    printf("Enter a multidimensional array for rhythm, \
            \n(1 = 1 dim, 2 = 2 dim, 3 = 3 dim, 4 = 4 dim): ");
    scanf("%d",&dims);

    switch(dims)
    {
    case 1:
        printf("you chose one dim\n");
        printf ("\n");
        rhyreturnarray = rhy1Dinput_gsl(rhyinputarray);
        break;
    case 2:
        printf("you chose two dims\n");
        printf ("\n");
        rhyreturnarray = rhy2Dinput_gsl(rhyinputarray);
        break;
    case 3:
        printf("you chose three dims\n");
        printf ("\n");
        rhyreturnarray = rhy3Dinput_gsl(rhyinputarray);
        break;
    case 4:
        printf("you chose four dims\n");
        printf ("\n");
        rhyreturnarray = rhy4Dinput_gsl(rhyinputarray);
        break;
    default:
        printf("you did not choose any dims\n");
    }
//**************************************
//********  PRINT RHYTHMS **************
//**************************************

    numrhys = 1;  //note problem passing to cscore2
    do
    {
        printf(" %4.6f ",rhyreturnarray[numrhys]);/*prints numbers */
        numrhys++;
    }
    while (rhyreturnarray[numrhys]>0.00001); //problem with floating pt.
    //cannot just use >0

//**************************************
//********  CSCORE (from frontends) ****
//**************************************

    result = csoundInitialize(&argc, (char ***)csvector, 0);

    if (result < 0)
    {
        fprintf(stderr, "Could not initialize the Csound library.\n");
        return -1;
    }

    cs = csoundCreate(NULL);
    if (cs == NULL)
    {
        fprintf(stderr, "Could not instantiate Csound.\n");
        return -1;
    }

    if (argc < 2)
    {
        fprintf(stderr, "Insufficient arguments: must provide an input filename.\n");
        return -1;
    }

    if (!(insco = fopen(csvector[0], "r")))
    {
        fprintf(stderr, "Cannot open the input score '%s'\n", csvector[0]);
        return -1;
    }

    if (argc < 3)
    {
        if (!(outsco = fopen(csvector[1], "w")))
        {
            fprintf(stderr, "Cannot open the output score '%s'\n", csvector[1]);
            return -1;
        }
    }

    csoundInitializeCscore(cs, insco, outsco);
    cscore2(cs, rhyreturnarray, pitchreturnarray, velocityreturnarray, numpitches, numrhys);

    fclose(insco); //may not be needed
    fclose(outsco); //may not be needed

//**************************************
//********  CSOUND PLAYER **************
//**************************************

    //reinitialize these values for use with csound compiler
    //from cscore use above
    /*
        //clear the argv array used above for cscore
    //    memset( argv, 0, sizeof(argv) );  //use memory.h
    const char *argv[] = {"csound", "jstest.orc", "jstest.sco"};
        argc = 3;
        //uses also .csoundrc for additional command line parameters
        //be sure to set the environment variable path for CSOUNDRC

     //initialise Csound library
        int myinit = csoundInitialize(&argc, (char ***)argv, 0);
        //Debug:printf("\n ***myinit result = %d", myinit);

        //Creates an instance of Csound.
        cs = csoundCreate(NULL);

        //compile and test the result
        result= csoundCompile(cs, 3, (char **)argv);
        //Debug:printf("*****compile result = %d", result);

        //int perftest = csoundPerformKsmps(csound);
        //Debug:printf("\n *******perftest = %d\n", perftest);

    	if(!result)
    	{
    	 while(csoundPerformKsmps(cs)==0){
           }

    	}
        csoundDestroy(cs);


        return result;
    //=================================//end play .csd
    */
    printf("\n");
    printf("\n...saved to %s", csvector[1]);
    printf ("\n*****numpitches = %d *****", numpitches-1);//DEBUG
    printf("\n******numvelocities = %d *****", numvelocities-1);//DEBUG
    printf("\n******numrhys = %d *****", numrhys-1);//DEBUG

////***************************************
////********  SAVE TO MID FILE OPTION *****
////***************************************
savetomidiswitch:
    printf("\n");
    printf("\nSave to MIDI File? (y)es, (n)o: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        exportmidi(pitchreturnarray, numpitches-1, velocityreturnarray, rhyreturnarray);
    case 'n':
        goto final;
    default:
        break;
        printf("\n...enter (y)es, or (n)o: \n");
        goto savetomidiswitch;
    }
final:

    printf("\nbye\n");

    return 0;

}
