/*
THIS CODE USES THE GSL LIBRARY
Must link using libgsl.so
*/

#include <stdlib.h>
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include </opt/csound516/Mygsl.h>
#include <time.h>
#include <sys/time.h>

struct statvars
{
    int structnum;
    double retdistval; // ran num returned from distribution
    double distinput; // distribution input
    double pdf; //pdf
    int midipitch; // midi pitch
};


int *pitchsetup(int *thereturnarray)
{
    //mem ptrs for ran gen
    const gsl_rng_type * T;
    gsl_rng * r;

    int i;
    double u, Y; //retrun values of struct Mygsl_pdfs
    float minpdf;

    char c;
    int *s = thereturnarray;
    int l, h, x, f;
    int numpitches, pitchdist, distcounter;

    PDFS astruct;

    struct statvars **ArrayofStructs = NULL; //declare ptr to array of structs
    struct timeval  t1; //for unix gettimeofday to seed rng

    printf("\t\t\t***** PITCHES *****");
    printf("\n");

//***************************************
//********  USER INTERFACE **************
//***************************************
first:
    s=(int*)calloc(1000,sizeof(int));/*pointer 's' is allocated 1000
				  integers and treated like an array */

    //allocate memory for 1 statvars struct
    ArrayofStructs = (struct statvars *)calloc(1, sizeof(struct statvars));

    /* create a generator chosen by the
       environment variable GSL_RNG_TYPE.
       In this case using 'taus' algo */
    gsl_rng_env_setup();
    T = gsl_rng_taus; //use taus algo as ran gen
    r = gsl_rng_alloc (T);  //alloc mem, see free below

    printf("\n");
    printf("How many pitches? (1000 MAX):");
    scanf("%d",&numpitches);

start:
    printf("\nEnter lowest midi note value for a pitch:");
    scanf("%d", &l);
    printf("\nEnter highest midi note value for a pitch:");
    scanf("%d", &h);
    printf("\n");
    if(h-1<=1) /*loop placed here as a precaution*/
    {
        goto start;
    }
    h=h+1;/* highest no. gets plus one */

//***************************************
//*****  SELECT RANDOM DISTRIBUTION *****
//***************************************
    printf("\n");
    printf("\t\t\t***** PITCH DISTRIBUTIONS *****");
    printf("\n1 = gaussian, 2 = gaussian_tail, 3 = exponential, 4 = laplace\
            \n5 = exppow, 6 = cauchy, 7 = rayleigh, 8 = rayleigh_tail\
            \n9 = landau, 10 = levy, 11 = levy_skew, 12 = gamma\
            \n13 = flat, 14 = lognormal, 15 = chisq, 16 = fdist\
            \n17 = tdist, 18 = beta, 19 = logistic, 20 = pareto\
            \n21 = Weibell, 22 = gumbel1, 23 = gumbel2, 24 = poisson\
            \n25 = binomial, 26 = negative_binomial, 27 = pascal\
            \n28 = geometric, 29 = hypergeometric, 30 = logarithmic\
            \n\nEnter a distribution for pitches:  ");
    scanf("%d",&pitchdist);

//------------- I N F O ------------------
//        For pitches and velocities, the Mygsl_pdfs struct from Mygls.h
//        will return a pdf value.  This allows for limiting pitches
//        which fall only within a given probability density function,
//        assuring that a particular distribution is more accurate.
//
//        If uncertain of the kinds of pdfs for a distribution, it's
//        best to enter o.o for minpdf input, then select 'stats' below
//        to see what types of pdfs are being returned.  Then try the
//        distrubution again but enter an appropirate minimum value
//        for PDF.
//------------- E N D  I N F O ------------------
        printf("\nEnter a minimum acceptable value for PDF (ex. 0.5).");
        printf("\nSome dists (ex. Levy & Levy_Skew) return 0.0 PDF,");
        printf("\nand Landau has a low PDF 0.1. :  ");
        scanf("%f",&minpdf);

    distcounter = 1;

//***************************************
//********  MAIN PITCHES LOOP ***********
//***************************************
    for (x=1; x<=numpitches; x++)
    {
next:
        gettimeofday(&t1, NULL);
        gsl_rng_default_seed = t1.tv_usec * t1.tv_sec; //use time as gen seed

        //****GSL. Get a random distribution****
        //****and reject if PDF percent is too low*****
        astruct = Mygsl_pitch_dists(pitchdist, distcounter, r);
        u = astruct.randistribution;
        Y = astruct.rand_pdf;

        //NOTE: odd problem here - if pdf is high, liklihood of
        //returning 'f' below within ranges is low.  Needs more work.
        if(Y< minpdf)
        {
            distcounter++;
            if (distcounter > 5000)
            {
                printf("\n...try adjusting the pitch dist input (sigma, etc.)");
                printf("\n...or the minimum PDF.\n");
                goto first;
            }
            goto next;
        }

        f=(int)(u*h-1)+1; //limits nums between ranges

        if(f<l || f>h-1)  /*avoid out of bounds nums */
        {
            if (distcounter > 3000)
            {
            free (ArrayofStructs);
            free(s);
            gsl_rng_free (r);
            printf("\n The distribution is not generating numbers\
                \n within the ranges specified.  Try entering\
                \n different input values for the distribution. \n");
            goto first;
            }
        distcounter++;
        goto next;
        }

        /* add ONE element to the array */
        ArrayofStructs= (struct statvars **)realloc(ArrayofStructs, (x+1 ) * sizeof(struct statvars *));
        /* allocate memory for x number of statvars */
        ArrayofStructs[x] = (struct startvars *)malloc(sizeof(struct statvars));
        //insert vars into local structure, each for loop pass
        ArrayofStructs[x]->structnum = x;
        ArrayofStructs[x]->retdistval = astruct.randistribution;
        ArrayofStructs[x]->distinput = astruct.rannum;
        ArrayofStructs[x]->pdf = astruct.rand_pdf;
        ArrayofStructs[x]->midipitch = f;
        s[x]= f;
        distcounter++;
    } //end for loop

    printf("\n...from gsl pitches.c\n");
    for (x=1; x<=numpitches; x++)
    {
        printf("%4d",s[x]);/*prints numbers */
    }
    printf("\n");

//****************************************
//********  User Switch to Try Again *****
//****************************************
    //switch with statistical module
userswitch:
   // printf("\n");
    printf("\nTry again? (y)es, (n)o, or (s)tats: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        free(s); //free memory
        free(ArrayofStructs);
        gsl_rng_free (r);
        goto first;

    case 'n':
        goto last;
    case 's':
        printf ("\nret a value for input of (rng) = (pdf) for pitch:(midi num).\n\n");
        for (x=1; x<=numpitches; x++)
        {
            printf ("ret(%f) at %f for value of %9.4f=pdf(%f). Pitch: %d\n",
                    ArrayofStructs[x]->retdistval, ArrayofStructs[x]->retdistval,
                    ArrayofStructs[x]->distinput, ArrayofStructs[x]->pdf, ArrayofStructs[x]->midipitch);
        }
        goto userswitch;

    default:
        break;
    } //ends switch

last:
    free(ArrayofStructs); //free memory
    gsl_rng_free (r);

    return s;
} //ends pitches.c
