#include <stdlib.h>
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>

//NOT a 3d array with page, row, col
//but a 2d array with 3 cols
#define ASTRUCT_MAXROWS 1
#define ASTRUCT_MAXCOLS 1

extern double Mygsl_rhy_dists(int distribnumber, int counter, const gsl_rng * r);

struct the3dstruct
{
    int structnum;
    int nrows;
    int ncols;
    float a3dstruct[ASTRUCT_MAXROWS][ASTRUCT_MAXCOLS];
};

//------------- I N F O ------------------
//function takes in a two dimensional array, with three cols
//and the number of triads to be returned
//
//2d array triads are each loaded into a struct
//ex. (0,0), (0,1). and (0,2)
//
//then an array of those structs is created
//then randomized, returning a 2 dim array, with 3 cols
//of the number of triads
//------------- E N D  I N F O ------------------

//NOT a 3d array with page, row, col
//but a 2d array with 3 cols
float *random3dstructs(int inrows, int incols,
    float **input3darray, int returnrows,
    int returncols, int rhydistrib)
{
    char c;
    //mem ptrs for ran gen
    const gsl_rng_type * T;
    gsl_rng * r;

    int spinrows = returnrows;
    int spincols = returncols;
    int spincount = returnrows * returncols;
    int hh = inrows * incols;
    int h; //highest number
    int l = 1; //lowest number
    int f, i, j, k, nn, x, y, oddevennumrange;
    int rhydist = rhydistrib;
    int distcounter;

    float **s = NULL;  //the local 2d array
    float *sPool, *curS_Ptr; //for calloc
    float *ss = NULL; //local 1d arrays
    int *pp = NULL;
    int *tt = NULL;

    double u; //value returned from Mygsl_rhy_dists

    struct the3dstruct **Arrayof3dStructs = NULL;
    struct timeval  t1; //structs for rand using time

first:

    distcounter = 1;  //sets or resets counter
//*****************************************
//********  ALLOCATE MEMORY ***************
//*****************************************

    /* create a generator chosen by the
       environment variable GSL_RNG_TYPE.
       In this case using 'taus' algo */
    gsl_rng_env_setup();
    T = gsl_rng_taus; //use taus algo as ran gen
    r = gsl_rng_alloc (T);  //alloc mem, see free below

//allocate local initialized 2d array memory, with 3 cols.
//NOTE: use of ptr pool for ease of freeing arrays later in the func
    s=(float **) calloc(spinrows+1, sizeof(float*));
    sPool = (float*)calloc(spincols*spinrows,sizeof(float));
    curS_Ptr=sPool;
    for (i = 0; i<spinrows+1; i++)
    {
        s[i]= curS_Ptr;
        curS_Ptr += spincols;
    }

    //allocate local initialized 1d array memory
    //Note: ss is different here from rhythm2d.c
    //if allocate only spincount, free(ss) gives error
    ss=(float *) calloc(spincount+1, sizeof(float*));
    pp=(int *) calloc(spincount, sizeof(int*));
    tt=(int *) calloc(spincount, sizeof(int*));

//allocate memory for one 3d struct
    Arrayof3dStructs = (struct the3dstruct *)calloc(1, sizeof(struct the3dstruct));
//allocate memory for the inrows*incols num of 3dstructs
    h= inrows*incols;
    for (x=0; x<=h; x++)
    {
        /* add ONE element to the array */
        Arrayof3dStructs= (struct the3dstruct **)realloc(Arrayof3dStructs, (x + 1) * sizeof(struct the3dstruct *));
        /* allocate memory for x number of the3dstructs */
        Arrayof3dStructs[x] = (struct the3dstruct *)malloc(sizeof(struct the3dstruct));
    }

//load data from incoming 2d array to local array of structs
//structnums begin with '1' for ease of counting and printing,
//but structs begin with '0'
    x=0;
    y=1;
    for (j = 0; j<inrows; j++)
    {
        for (k = 0; k<incols; k++)
        {
            Arrayof3dStructs[x]->a3dstruct[0][0] = input3darray[j][k];
            Arrayof3dStructs[x]->structnum = y;
            x++;
            y++;
        }
    }

//load oddeven numbers into local 1d array (1,4,7,10,13,16...)
//hh=inrows, incols
    oddevennumrange=1;
    for (x=1; x<=hh; x+=3)
    {
        pp[oddevennumrange] = x;
        oddevennumrange++;
    }
    h = oddevennumrange;//highest equals the number of values + 1 (from oddevennumrange++ above)

//spincount = returnrows * returncols
    for (x=1; x<=spincount/3; x++)
    {
next:
        gettimeofday(&t1, NULL);
        gsl_rng_default_seed = t1.tv_usec * t1.tv_sec; //use time as gen seed

        //****GSL. Get a random distribution****
        u = Mygsl_rhy_dists(rhydist, distcounter, r);

        f = (int)(u*h-1)+1; //limits between ranges

                if (distcounter > 3000)
        {
            free (Arrayof3dStructs);
            free(*s);
            free(s);
            free(ss);
            free(pp);
            free(tt);
            gsl_rng_free (r);
            printf("\n The distribution is not generating numbers\
                \n within the ranges specified.  Try entering\
                \n different input values for the distribution. \n");
            goto first;
        }


        if(h == 1)   //avoids rare case where highest = 1
        {
            if(f<l || f>h)
            {
                distcounter++;
                goto next;
            }
        }
        else
        {
            if(f<l || f>h-1)  /*avoid out of bounds nums */
            {
                distcounter++;
                goto next;
            }
        }
        tt[x]= pp[f];
        distcounter++;

    } //end for loop
//+++++++++++++++++++++

//*********************************************************
//********  LOADING and PRINTING spun array ***************
//*********************************************************

// Fill All slots
// uses the randomized numbers, from above
    x=0;
    for(i=1; i<=spincount/3; i++)
    {
        nn=tt[i];
        x=x+1;
        ss[x]=Arrayof3dStructs[nn-1]->a3dstruct[0][0]; //1, 4, 7, 10, 13, 16 ...
        x=x+1;
        ss[x]=Arrayof3dStructs[nn]->a3dstruct[0][0];   //2, 5, 8, 11, 14, 17...
        x=x+1;
        ss[x]=Arrayof3dStructs[nn+1]->a3dstruct[0][0]; //3, 6, 9, 12, 15, 18...
    }


// loads the 1d array into a 2d array to return to the
// RHY3DINPUT func below
    printf("\n...printing the spincount of 2d spun structs (from rand3dstructs)...\n");
    i=0;
    for (j = 0; j<spinrows; j++)
    {
        for (k = 0; k<spincols; k++)
        {
            i++;
            s[j][k] = ss[i];
             printf(" %4.6f ", s[j][k]);
        }
    }

    printf("try again? (y)es, (n)o: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        //free ALL memory allocated so far
        for (x=0; x<=h; x++)
        {
            free(Arrayof3dStructs[x]);
        }
        free (Arrayof3dStructs);
        free(*s);//NOTE: this how ptr pool helps with freeing memory
        free(s);
        free(ss);
        free(pp);
        free(tt);
        gsl_rng_free (r);
        goto first;
        break;
    case 'n':
        printf("\nbye");
        break;
    default:
        break;
    }//ends switch
    printf("\n");

    for (x=0; x<=h; x++)
    {
        free(Arrayof3dStructs[x]);
    }
    free (Arrayof3dStructs);
    free(ss);
    free(pp);
    free(tt);
    gsl_rng_free (r);

    return s; //return only the local array of randomized nums

} //end random3dstructs

//++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++

//******   RHY3DINPUT    ******

//++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++

//NOT a 3d array with page, row, col
//but a 2d array with 3 cols
float *rhy3Dinput_gsl(float *rhy3Dreturnarray)
{
    char c;
    float **struct3dinputarray = NULL; //local 2d arrays with 3 cols
    float **struct3dreturnarray = NULL;
    float  *struct1dreturnarray = NULL;
    int i, j, k, nn, x;
    int rows, spinrows;  //number of input rows and returned rows
    int cols = 3;  //one input pair (for 3d leave cols as "3")
    int spincols = 3; //one returned group of three
    float *inputPool, *returnPool; //for calloc
    float *curInputPtr, *curReturnPtr; //for calloc

    int rhydist;//the distribution selection

//**************************************
//********  ENTER VALUES ***************
//**************************************
//fill a 2d array, with 3 cols, to send to rand3dsturcts function above
start:
    printf("\n");
    printf("Enter the number of triads: ");
    scanf("%d",&rows);
//printf("\nThe number of pairs is: %d", rows); //DEBUG

//allocate regular 2d array memory
//keep 2d array as 3 cols
//But rows * num of pairs are needed for input
//NOTE: use of ptr pool for ease of freeing arrays later in the func
    struct3dinputarray=(float **) calloc(rows, sizeof(float*));
    inputPool = (float*)calloc(cols*rows,sizeof(float));
    curInputPtr=inputPool;
    for (i = 0; i<rows; i++)
    {
        struct3dinputarray[i]= curInputPtr;
        curInputPtr += cols;
    }

//load the 2d array with rhythmic pairs, ex:
//(0,0)=0.33, (0,1)=0.33, (0,2)=0.34
//(1,0)=0.5, (1,1)=0.25, (1,2)=0.25 etc.
    x=1;
    for (j = 0; j<rows; j++)
    {
        for (k = 0; k<cols; k++)
        {
            printf("Traid #%d, enter value #%d: ", j+1, x);
            scanf("%f",&struct3dinputarray [j][k]);
            x++;
        }
    }

//------------- I N F O ------------------
// todo:  add error checking here to make sure each group of three
// sums to a whole number.  In that way "Beats" returned
// will be clear.
// However could also just leave it up to the user to manage the
// the entry of values, so for example syncopation, or groups
// of three larger than one beat(quarter) could result
//------------- E N D  I N F O ------------------
    printf("\n...here's the list \n");
    for (j = 0; j<rows; j++)
    {
        for (k = 0; k<cols; k++)
        {
            printf("%2.4f\n",  struct3dinputarray[j][k]); // Displaying the Array
        }
    }

//switch for another spin before returning values
    printf("try again? (y)es, (n)o: ");
    while (getchar()!='\n');
    scanf("%c", &c);
    switch(c)
    {
    case 'y':
        //free 2d array memory
//NOTE: this how ptr pool helps with freeing memory
        free(*struct3dinputarray);
        free(struct3dinputarray);
        goto start;
        break;
    case 'n':
        goto next;
        break;
    }//ends switch
    printf("\n");
//***************************************
//********  RETURN VALUES ***************
//***************************************
next:
//number of groups of three to be returned
    printf("\nEnter the number of triads to be returned: ");
    scanf("%d",&spinrows);

//allocate 1d array memory
    struct1dreturnarray=(float **) calloc((spinrows*spincols+1), sizeof(float*));

//allocate 2d array memory, with three cols
//NOTE: use of ptr pool for ease of freeing arrays later in the func
    struct3dreturnarray=(float **) calloc(spinrows, sizeof(float*));
    returnPool = (float*)calloc(spincols*spinrows,sizeof(float));
    curReturnPtr = returnPool;
    for (i = 0; i<spinrows; i++)
    {
        struct3dreturnarray[i]= curReturnPtr;
        curReturnPtr += spincols;
    }

//***************************************
//*****  SELECT RANDOM DISTRIBUTION *****
//***************************************

    printf("\n");
    printf("\t\t\t***** RHYTHM DISTRIBUTIONS *****");
    printf("\n1 = gaussian, 2 = gaussian_tail, 3 = exponential, 4 = laplace\
            \n5 = exppow, 6 = cauchy, 7 = rayleigh, 8 = rayleigh_tail\
            \n9 = landau, 10 = levy, 11 = levy_skew, 12 = gamma\
            \n13 = flat, 14 = lognormal, 15 = chisq, 16 = fdist\
            \n17 = tdist, 18 = beta, 19 = logistic, 20 = pareto\
            \n21 = Weibell, 22 = gumbel1, 23 = gumbel2, 24 = poisson\
            \n25 = binomial, 26 = negative_binomial, 27 = pascal\
            \n28 = geometric, 29 = hypergeometric, 30 = logarithmic\
            \n\nEnter a distribution for rhythm:  ");
    scanf("%d",&rhydist);

//***************************************
//********  CALL random2dstructs_gsl ********
//***************************************
    struct3dreturnarray = random3dstructs(rows, cols, struct3dinputarray, spinrows, spincols, rhydist);

// convert the 2d array from random3dstructs function to a 1d array
// to RETURN to main, for use with Cscore
//printf("testing return\n");
    x=0;
    for (j = 0; j<spinrows; j++)
    {
        for (k = 0; k<spincols; k++)
        {
            x++;
            struct1dreturnarray[x] = struct3dreturnarray[j][k];
            //printf("%d = %4.6f\n", x, struct1dreturnarray[x]); //DEBUG
        }
    }

    //free 2d array memory
//NOTE: this how ptr pool helps with freeing memory
    free(*struct3dinputarray);
    free(struct3dinputarray);

    free(*struct3dreturnarray);
    free(struct3dreturnarray);

    return struct1dreturnarray;   //returns a 1d array of rhythms

} //end rhy3dinput
